	processor 6502


;*
;*	Zero page locations used
;*

src 	equ $fb
srch	equ $fc
dst		equ $fd
dsth	equ $fe

aux		equ $ff
bitmask	equ	$00
bits	equ $01
case	equ $02


;*
;*	Font data
;*
	org     $2000

	incbin "miniascii.bin"


;*
;*	Jump vectors
;*
	org		$2200

	jmp	Init
	jmp	Clear
	jmp CharOut


;*
;*	Init VIC-I
;*

Init	subroutine

	pha
	txa
	pha
	tya 
	pha

	sei


;	Check video standard (PAL or NTSC)

.1
	lda $9004	; wait for raster line 0
	beq .1
.2
	lda $9004	; wait for last raster line
	beq .3
	tay
	bne .2
.3

	ldx	#$03	; NTSC screen position
	lda #$1f
	cpy #$82	; $82 = NTSC, $9b = PAL
	beq	.4
	ldx #$0a	; PAL screen position
	lda	#$2c
.4


;	Set up screen position, screen size, character generator, screen matrix, background & border color

	stx $9000	; set screen position
	sta $9001

	lda #$18	; screen width: 24 characters x 8 pixels = 192 pixels
	sta	$9002
	lda #$15	; height 10 characters x 16 pixels = 160 pixels, bit 0 = 16 pixel character mode
	sta $9003

	lda #$cc	; screen matrix at $1000, bitmap at $1100-$1fff (char codes $10-$ff used)
	sta $9005

	lda #$08	; black screen, black border
	sta $900f


;	Init bitmap, $a0 consecutive bytes in each column

	lda #$10	; first character code at top left corner
	ldy #$00
.nextline
	ldx #$18
.nextchar
	sta $1000,y
	clc
	adc #$0a	; next character on line
	iny
	beq .lastchar
	dex
	bne .nextchar
	sec
	sbc	#$ef	; first character of next line
	bne .nextline
.lastchar

	pla 
	tay
	pla
	tax 
	pla 
	
	cli
	rts


;*
;*	Clear screen
;*

Clear	subroutine

	pha
	txa
	pha
	tya 
	pha


;	Init color RAM to white

	ldy	#$00
	lda #$01
.1
	sta $9400,y
	iny
	bne .1


;	Clear bitmap

	lda #$11	; start of bitmap ($1100)
	sta dsth

	ldx	#$0f	; number pages to clear
	lda	#$00	; bitmap init value

	ldy	#$00
	sty	dst

.2
	sta (dst),y
	iny
	sta (dst),y
	iny
	sta (dst),y
	iny
	sta (dst),y
	iny
	bne .2

	inc dsth
	dex
	bne	.2

	pla 
	tay
	pla
	tax 
	pla 
	
	rts


;*
;*	CharOut, x = x-coordinate, y = y-coordinate, a = ASCII character 0-127
;*

CharOut	subroutine

;	check x-/y-coordinate bounds

	cpx	#$40
	bcc	.okx
	rts
.okx
	cpy #$20
	bcc	.oky
	rts
.oky

	pha
	and	#$7f
	sta aux
	txa
	pha
	tya 
	pha

	lda aux		; low byte of character data in font
	and #$1f
	asl
	asl
	asl
	sta src

	lda aux		; $00-$3f or $40-$7f to carry
	asl
	asl
	lda #$20
	adc #$00
	sta srch

	lda aux		; upper/lowercase (or ctrl/numbers)
	and #$20
	asl
	asl
	sta case


;	lookup bitmap location for x, y and calculate shifts

	txa			; x = x * 2 (for xtable offset), aux = x * 3 and 7 (amount of shifts)
	sta aux
	asl
	tax
	clc
	adc	aux
	and	#$07
	sta	aux

	lda	xtable,x
	clc
	adc	ytable,y
	sta dst
	lda xtable+1,x
	adc #$00
	sta dsth


;	output character

	ldx	aux			; get matching bit mask for exact x-coordinate
	lda	masktable,x
	sta bitmask

	ldy	#$04
.nextrow
	lda (src),y		; get font data
	bit case
	bpl .ucase1
	asl
	asl
	asl
	asl
.ucase1
	and #$f0

	ldx	aux			; shift bits into place
	beq .1
.2
	lsr 
	dex
	bne .2
.1
	sta bits

	lda (dst),y		; draw to bitmap with mask
	and	bitmask
	ora bits
	sta (dst),y

	dey				; next pixel row
	bpl .nextrow

	ldx aux			; check for pixels slipping to next column in bitmap
	lda	rightmask,x
	cmp	#$ff
	beq	.done		; if none then character is ready
	sta bitmask

	lda	dst			; move to next column in bitmap
	adc	#$a0
	sta dst
	bcc .noc
	inc dsth
.noc

	ldy	#$04
.lasttwo
	lda (src),y		; get font data
	bit case
	bpl .ucase2
	asl
	asl
	asl
	asl
.ucase2
	and #$f0

	ldx aux			; shift bits into place
.3
	asl
	inx
	cpx #8
	bne .3

	sta bits

	lda (dst),y		; draw to bitmap with mask
	and	bitmask
	ora bits 
	sta (dst),y

	dey				; next pixel row
	bpl .lasttwo

.done
	pla 
	tay
	pla
	tax 
	pla 
	
	rts


;	character masks

masktable
	dc.b	%00011111, %10001111, %11000111, %11100011, %11110001, %11111000, %11111100, %11111110
rightmask
	dc.b	%11111111, %11111111, %11111111, %11111111, %11111111, %11111111, %01111111, %00111111

;	multiply table y*5 (bitmap offset to row, y = 0-31)

ytable
	dc.b	$00*5, $01*5, $02*5, $03*5, $04*5, $05*5, $06*5, $07*5
	dc.b	$08*5, $09*5, $0a*5, $0b*5, $0c*5, $0d*5, $0e*5, $0f*5
	dc.b	$10*5, $11*5, $12*5, $13*5, $14*5, $15*5, $16*5, $17*5
	dc.b	$18*5, $19*5, $1a*5, $1b*5, $1c*5, $1d*5, $1e*5, $1f*5


;	column to bitmap address table (x = 0-63)

xtable
	dc.w	$1100+($00*3/8)*$a0, $1100+($01*3/8)*$a0, $1100+($02*3/8)*$a0, $1100+($03*3/8)*$a0
	dc.w	$1100+($04*3/8)*$a0, $1100+($05*3/8)*$a0, $1100+($06*3/8)*$a0, $1100+($07*3/8)*$a0
	dc.w	$1100+($08*3/8)*$a0, $1100+($09*3/8)*$a0, $1100+($0a*3/8)*$a0, $1100+($0b*3/8)*$a0
	dc.w	$1100+($0c*3/8)*$a0, $1100+($0d*3/8)*$a0, $1100+($0e*3/8)*$a0, $1100+($0f*3/8)*$a0
	dc.w	$1100+($10*3/8)*$a0, $1100+($11*3/8)*$a0, $1100+($12*3/8)*$a0, $1100+($13*3/8)*$a0
	dc.w	$1100+($14*3/8)*$a0, $1100+($15*3/8)*$a0, $1100+($16*3/8)*$a0, $1100+($17*3/8)*$a0
	dc.w	$1100+($18*3/8)*$a0, $1100+($19*3/8)*$a0, $1100+($1a*3/8)*$a0, $1100+($1b*3/8)*$a0
	dc.w	$1100+($1c*3/8)*$a0, $1100+($1d*3/8)*$a0, $1100+($1e*3/8)*$a0, $1100+($1f*3/8)*$a0
	dc.w	$1100+($20*3/8)*$a0, $1100+($21*3/8)*$a0, $1100+($22*3/8)*$a0, $1100+($23*3/8)*$a0
	dc.w	$1100+($24*3/8)*$a0, $1100+($25*3/8)*$a0, $1100+($26*3/8)*$a0, $1100+($27*3/8)*$a0
	dc.w	$1100+($28*3/8)*$a0, $1100+($29*3/8)*$a0, $1100+($2a*3/8)*$a0, $1100+($2b*3/8)*$a0
	dc.w	$1100+($2c*3/8)*$a0, $1100+($2d*3/8)*$a0, $1100+($2e*3/8)*$a0, $1100+($2f*3/8)*$a0
	dc.w	$1100+($30*3/8)*$a0, $1100+($31*3/8)*$a0, $1100+($32*3/8)*$a0, $1100+($33*3/8)*$a0
	dc.w	$1100+($34*3/8)*$a0, $1100+($35*3/8)*$a0, $1100+($36*3/8)*$a0, $1100+($37*3/8)*$a0
	dc.w	$1100+($38*3/8)*$a0, $1100+($39*3/8)*$a0, $1100+($3a*3/8)*$a0, $1100+($3b*3/8)*$a0
	dc.w	$1100+($3c*3/8)*$a0, $1100+($3d*3/8)*$a0, $1100+($3e*3/8)*$a0, $1100+($3f*3/8)*$a0



;*
;*	Test
;*
	org		$2800

Test	subroutine
	jsr	$2200	; Init
	jsr $2203	; Clear

	ldy	#0
	ldx	#0
loop
	lda #$5f	; Cursor character
	jsr $2206	; CharOut

	sty	ycoord 
	stx xcoord 

	jsr $ffe4	; Kernal GETIN
	cmp #$00
	beq	loop
	sta $3000

	ldy	ycoord
	ldx	xcoord

	pha
	lda	#$20	; Clear cursor
	jsr	$2206	; CharOut
	pla

	cmp #$0d
	beq	.return
	cmp #$1d
	beq .cursorright
	cmp #$9d
	beq .cursorleft
	cmp #$14	; backspace
	beq .cursorleft
	cmp #$91
	beq .cursorup
	cmp #$11
	beq .cursordown
	cmp	#$13
	beq .cursorhome
	cmp	#$93
	beq .clearhome

	cmp	#$00
	bpl	.notcap
	and #$7f	; uppercase
	bpl	.cap
.notcap
	cmp #$41	; lowercase
	bcc	.cap
	clc
	adc	#$20

.cap
	jsr	$2206	; CharOut

.cursorright
	inx
	cpx #64
	bne	loop
.return
	ldx #$00
.cursordown
	iny
	cpy #$20
	bne loop
	dey
	jmp loop
.cursorleft
	dex
	bpl	loop
	ldx #$00
	beq loop
.cursorup
	dey
	bpl	loop
	ldy #$00
	beq	loop

.clearhome
	jsr $2203	; Clear
.cursorhome
	ldx	#$00
	ldy	#$00
	beq loop

ycoord
	dc.b	0
xcoord
	dc.b	0